module Vips
  # The binary math operation to perform on an image. See {Image#math2}.
  #
  # * ':pow' pow()
  # * ':wop' pow(), but with the arguments reversed

  class OperationMath2 < Symbol
  end
end
